@@ -52,7 +52,9 @@ main (gint argc, gchar *argv[])
   DeeModel *model;
   
   g_type_init ();
+#if !GLIB_CHECK_VERSION(2, 32, 0)
   g_thread_init (NULL);
+#endif
 
   model = dee_shared_model_new ("com.canonical.Dee.Model.Example");
   dee_model_set_schema (model, "i", "s", NULL);
