@@ -65,8 +65,10 @@ main (gint argc, gchar *argv[])
   DeeModel *model;
   const gchar *model_name;
   
-  g_type_init (); 
+  g_type_init ();
+#if !GLIB_CHECK_VERSION(2, 32, 0)
   g_thread_init (NULL);
+#endif
 
   if (argc < 2)
     model_name = "com.canonical.Dee.Model.Example";
