@@ -975,6 +975,20 @@ dee_model_insert_row_before (DeeModel      *self,
   return (* iface->insert_row_before) (self, iter, row_members);
 }
 
+/* Translates DeeCompareRowFunc callback into DeeCompareRowSizedFunc */
+static gint
+dee_model_cmp_func_translate_func (GVariant **row1,
+                                   GVariant **row2,
+                                   gpointer data)
+{
+  gpointer *all_data = (gpointer*) data;
+  DeeCompareRowSizedFunc cmp_func = (DeeCompareRowSizedFunc) all_data[0];
+  gpointer user_data = all_data[1];
+  guint array_length = GPOINTER_TO_UINT (all_data[2]);
+
+  return cmp_func (row1, array_length, row2, array_length, user_data);
+}
+
 /**
  * dee_model_insert_row_sorted:
  * @self: The model to do a sorted insert on
@@ -1008,6 +1022,41 @@ dee_model_insert_row_sorted (DeeModel           *self,
   return (* iface->insert_row_sorted) (self, row_members, cmp_func, user_data);
 }
 
+/**
+ * dee_model_insert_row_sorted_with_sizes:
+ * @self: The model to do a sorted insert on
+ * @row_members: (array zero-terminated=1): An array of
+ *       #GVariants with type signature matching those of the
+ *       column schemas of @self. If any of the variants have floating
+ *       references they will be consumed.
+ * @cmp_func: (scope call): Callback used for comparison or rows
+ * @user_data: (closure): Arbitrary pointer passed to @cmp_func during search
+ *
+ * Inserts a row in @self according to the sorting specified by @cmp_func.
+ * If you use this method for insertion you should not use other methods as this
+ * method assumes the model to be already sorted by @cmp_func.
+ *
+ * Returns: (transfer none) (type Dee.ModelIter): A #DeeModelIter pointing to the new row
+ */
+DeeModelIter*
+dee_model_insert_row_sorted_with_sizes (DeeModel               *self,
+                                        GVariant              **row_members,
+                                        DeeCompareRowSizedFunc  cmp_func,
+                                        gpointer                user_data)
+{
+  gpointer all_data[3];
+
+  g_return_val_if_fail (DEE_IS_MODEL (self), NULL);
+
+  all_data[0] = cmp_func;
+  all_data[1] = user_data;
+  all_data[2] = GUINT_TO_POINTER (dee_model_get_n_columns (self));
+
+  return dee_model_insert_row_sorted (self, row_members,
+                                      dee_model_cmp_func_translate_func,
+                                      all_data);
+}
+
 /**
  * dee_model_insert_sorted:
  * @self: The model to do a sorted insert on
@@ -1102,6 +1151,55 @@ dee_model_find_row_sorted (DeeModel           *self,
                                      user_data, out_was_found);
 }
 
+/**
+ * dee_model_find_row_sorted_with_sizes:
+ * @self: The model to search
+ * @row_spec: (array zero-terminated=1): An array of
+ *       #GVariants with type signature matching those of the
+ *       column schemas of @self. No references will be taken on the variants.
+ * @cmp_func: (scope call): Callback used for comparison or rows
+ * @user_data: (closure): Arbitrary pointer passed to @cmp_func during search
+ * @out_was_found: (out): A place to store a boolean value that will be set when
+ *                 this method returns. If %TRUE then an exact match was found.
+ *                 If %FALSE then the returned iter points to a row just after
+ *                 where @row_spec would have been inserted.
+ *                 Pass %NULL to ignore.
+ *
+ * Like dee_model_find_row_sorted(), but uses DeeCompareRowSizedFunc and
+ * therefore doesn't cause trouble when used from introspected languages.
+ *
+ * Finds a row in @self according to the sorting specified by @cmp_func.
+ * This method will assume that @self is already sorted by @cmp_func.
+ *
+ * If you use this method for searching you should only use
+ * dee_model_insert_row_sorted() (or dee_model_insert_row_sorted_with_sizes())
+ * to insert rows in the model.
+ *
+ * Returns: (transfer none) (type Dee.ModelIter): If @out_was_found is set to
+ *           %TRUE then a #DeeModelIter pointing to the first matching row.
+ *           If it is %FALSE then the iter pointing to the row just after where
+ *           @row_spec_would have been inserted.
+ */
+DeeModelIter*
+dee_model_find_row_sorted_with_sizes (DeeModel                *self,
+                                      GVariant               **row_spec,
+                                      DeeCompareRowSizedFunc   cmp_func,
+                                      gpointer                 user_data,
+                                      gboolean                *out_was_found)
+{
+  gpointer all_data[3];
+
+  g_return_val_if_fail (DEE_IS_MODEL (self), NULL);
+
+  all_data[0] = cmp_func;
+  all_data[1] = user_data;
+  all_data[2] = GUINT_TO_POINTER (dee_model_get_n_columns (self));
+
+  return dee_model_find_row_sorted (self, row_spec,
+                                    dee_model_cmp_func_translate_func,
+                                    all_data, out_was_found);
+}
+
 /**
  * dee_model_find_sorted:
  * @self: The model to search
