@@ -47,6 +47,7 @@ static void test_no_transfer     (RowsFixture *fix, gconstpointer data);
 static void test_iter_backwards  (RowsFixture *fix, gconstpointer data);
 static void test_illegal_access  (RowsFixture *fix, gconstpointer data);
 static void test_sorted          (RowsFixture *fix, gconstpointer data);
+static void test_sorted_with_sizes (RowsFixture *fix, gconstpointer data);
 
 static void test_model_iter_copy (RowsFixture *fix, gconstpointer data);
 static void test_model_iter_free (RowsFixture *fix, gconstpointer data);
@@ -133,6 +134,13 @@ test_model_rows_create_suite (void)
               proxy_rows_setup, test_sorted, proxy_rows_teardown);
   g_test_add (TXN_DOMAIN"/Sorted", RowsFixture, 0,
               txn_rows_setup, test_sorted, txn_rows_teardown);
+
+  g_test_add (SEQ_DOMAIN"/Sorted/WithSizes", RowsFixture, 0,
+              seq_rows_setup, test_sorted_with_sizes, seq_rows_teardown);
+  g_test_add (PROXY_DOMAIN"/Sorted/WithSizes", RowsFixture, 0,
+              proxy_rows_setup, test_sorted_with_sizes, proxy_rows_teardown);
+  g_test_add (TXN_DOMAIN"/Sorted/WithSizes", RowsFixture, 0,
+              txn_rows_setup, test_sorted_with_sizes, txn_rows_teardown);
 }
 
 static void
@@ -560,3 +568,125 @@ test_sorted (RowsFixture *fix, gconstpointer data)
   g_assert (was_found);
   g_assert (result == hter);
 }
+
+static gint
+sized_cmp_col_0 (GVariant **row1, guint row1_length,
+                 GVariant **row2, guint row2_length, gpointer user_data)
+{
+  g_assert_cmpstr (user_data, ==, "test-user-data");
+  g_assert_cmpuint (row1_length, ==, 2);
+  g_assert_cmpuint (row2_length, ==, 2);
+  g_assert_cmpuint (row1_length, ==, row2_length);
+  return g_variant_get_int32 (row2[0]) - g_variant_get_int32 (row1[0]);
+}
+
+static void
+test_sorted_with_sizes (RowsFixture *fix, gconstpointer data)
+{
+  DeeModelIter *hter, *iter, *jter, *kter;
+  gboolean      was_found;
+  GVariant     *row_spec[2];
+
+  /* FINAL MODEL: [(28,s), (27,s), (26,s), (25,s)]
+   *           ~= [hter, iter, jter, kter] */
+
+  row_spec[0] = g_variant_new_int32 (0);
+  row_spec[1] = g_variant_new_string ("");
+  /* Test find() with an empty model. With NULL was_found arg */
+  iter = dee_model_find_row_sorted_with_sizes (fix->model, row_spec,
+                                               sized_cmp_col_0,
+                                               "test-user-data",
+                                               NULL);
+  g_assert (iter == dee_model_get_last_iter (fix->model));
+
+  /* Test find() with an empty model. With non-NULL was_found arg */
+  was_found = TRUE;
+  iter = dee_model_find_row_sorted_with_sizes (fix->model, row_spec,
+                                               sized_cmp_col_0,
+                                               "test-user-data",
+                                               &was_found);
+  g_assert (!was_found);
+  g_assert (iter == dee_model_get_last_iter (fix->model));
+
+  /* Insert the first row */
+  row_spec[0] = g_variant_new_int32 (27);
+  row_spec[1] = g_variant_new_string ("Sorta sorted");
+  iter = dee_model_insert_row_sorted_with_sizes (fix->model, row_spec,
+                                                 sized_cmp_col_0,
+                                                 "test-user-data");
+  g_assert (iter != dee_model_get_last_iter (fix->model));
+  g_assert (iter == dee_model_get_first_iter (fix->model));
+
+  /* Test append */
+  row_spec[0] = g_variant_new_int32 (25);
+  row_spec[1] = g_variant_new_string ("Sorta sorted");
+  kter = dee_model_insert_row_sorted_with_sizes (fix->model, row_spec,
+                                                 sized_cmp_col_0,
+                                                 "test-user-data");
+  g_assert (kter != dee_model_get_last_iter (fix->model));
+  g_assert (kter != dee_model_get_first_iter (fix->model));
+  g_assert (iter == dee_model_get_first_iter (fix->model));
+  g_assert (kter != iter);
+
+  g_assert_cmpint (2, ==, dee_model_get_n_rows (fix->model));
+  g_assert (kter == dee_model_next (fix->model, iter));
+
+  /* Test insert in between rows */
+  row_spec[0] = g_variant_new_int32 (26);
+  row_spec[1] = g_variant_new_string ("Sorta sorted");
+  jter = dee_model_insert_row_sorted_with_sizes (fix->model, row_spec,
+                                                 sized_cmp_col_0,
+                                                 "test-user-data");
+  g_assert (jter != dee_model_get_last_iter (fix->model));
+  g_assert (jter != dee_model_get_first_iter (fix->model));
+  g_assert (iter == dee_model_get_first_iter (fix->model));
+  g_assert (jter != iter);
+  g_assert (jter != kter);
+
+  g_assert (jter == dee_model_next (fix->model, iter));
+  g_assert (kter == dee_model_next (fix->model, jter));
+  g_assert (dee_model_get_last_iter (fix->model) == dee_model_next (fix->model, kter));
+
+  /* Test prepend */
+  row_spec[0] = g_variant_new_int32 (28);
+  row_spec[1] = g_variant_new_string ("Sorta sorted");
+  hter = dee_model_insert_row_sorted_with_sizes (fix->model, row_spec,
+                                                 sized_cmp_col_0,
+                                                 "test-user-data");
+  g_assert (hter == dee_model_get_first_iter (fix->model));
+  g_assert (iter == dee_model_next (fix->model, hter));
+
+  g_assert_cmpint (4, ==, dee_model_get_n_rows (fix->model));
+
+  /* Test find() again now that we have data in the model */
+  DeeModelIter *result;
+  row_spec[0] = g_variant_new_int32 (24);
+  row_spec[1] = g_variant_new_string ("");
+  result = dee_model_find_row_sorted_with_sizes (fix->model, row_spec,
+                                                 sized_cmp_col_0,
+                                                 "test-user-data", NULL);
+  g_assert (result == dee_model_get_last_iter (fix->model));
+  row_spec[0] = g_variant_new_int32 (28);
+  row_spec[1] = g_variant_new_string ("");
+  result = dee_model_find_row_sorted_with_sizes (fix->model, row_spec,
+                                                 sized_cmp_col_0,
+                                                 "test-user-data", NULL);
+  g_assert (result == hter);
+
+  /* Test find(). With non-NULL was_found arg */
+  was_found = FALSE;
+  row_spec[0] = g_variant_new_int32 (24);
+  row_spec[1] = g_variant_new_string ("");
+  result = dee_model_find_row_sorted_with_sizes (fix->model, row_spec,
+                                                 sized_cmp_col_0,
+                                                 "test-user-data", &was_found);
+  g_assert (result == dee_model_get_last_iter (fix->model));
+  row_spec[0] = g_variant_new_int32 (28);
+  row_spec[1] = g_variant_new_string ("");
+  result = dee_model_find_row_sorted_with_sizes (fix->model, row_spec,
+                                                 sized_cmp_col_0,
+                                                 "test-user-data", &was_found);
+  g_assert (was_found);
+  g_assert (result == hter);
+}
+
